// This example project highlights the use of the Linker API library. 
// compile with: /EHsc /link IntegralLibrary.lib
// This project is generated in MS Visual Studio 2015 using v120_xp platform toolset

#include "stdafx.h"
#include "windows.h"
#include "types.h"
#include "linkerapi_enum.h"
#include "DefaultEdid.h"
#include "linkerapi.h"

bool GetLinkStatus(uint8 *RdStatus);

int _tmain(int argc, _TCHAR* argv[])
{

	bool Status = false;
	uint8 LinkState[100];

	LINKER_VIDEOSTATUS vs;
	uint8 defaultEDID[256];
	uint8 sinkEDID[256];
	char fw_str[80];

	LeaveDeviceOpen();  // If using LeaveDeviceOpen() everything is much faster. 
						// Remember also to use DeviceClose(); before exit the program

	getFwVersion(fw_str);
	printf("%s\n", fw_str);

	memset(&vs, 0x00, sizeof(vs));
	Status = GetLinkStatus(LinkState);
	if (Status == false) {
		printf("\nError reading Link...\n");
		return(false);
	}

	getVideoFormat(&vs);
	printf("%s\n", vs.video_format);
	printf("%s\n", vs.video_outformat);
	printf("%s\n", vs.Csc);
	printf("%s\n", vs.CscO);
	printf("%s\n", vs.PixelClock);
	printf("%s\n", vs.OutPixelClock);
	printf("%s\n", vs.SPDIF);
	printf("%s\n", vs.VSI);
	printf("%s\n", vs.HDMIVSI);
	printf("%s\n", vs.AVI);
	printf("%s\n", vs.HDR);
	printf("%s\n", vs.HdrInput);
	printf("%s\n", vs.SPD);
	printf("%s\n", vs.HdcpIn);
	printf("%s\n", vs.HdcpOut);
	printf("%s\n", vs.DcIn);
	printf("%s\n", vs.DcOut);
	printf("%s\n", vs.MonitorName);

	//*****************************************************
	printf("Selecting Port B.\n");
	setActivePort(INPORT_B);
	printf("\nPress ENTER key to Continue\n");  getchar();
	//*****************************************************
	printf("Selecting Port A.\n");
	setActivePort(INPORT_A);
	printf("\nPress ENTER key to Continue\n");  getchar();
	//*****************************************************

	printf("\n\nReading active EDID.\n");
	Status = getEdid(defaultEDID);
	if (Status == true) {
		for (int i = 0; i<256; i++) {
			if (!(i % 16))
				printf("\n");
			printf("%02X ", defaultEDID[i]);
		}
	}
	else {
		printf("\nError reading edid values.\n");
	}

	//*****************************************************	
	printf("\n\nReading Sink EDID.\n");
	Status = getSinkEdid(sinkEDID);
	if (Status == true) {
		for (int i = 0; i < 256; i++) {
			if (!(i % 16))
				printf("\n");
			printf("%02X ", sinkEDID[i]);
		}
	}
	else {
		printf("\nError reading edid values.\n");
	}
	printf("\nPress ENTER key to Continue\n");  getchar();


	//*****************************************************
	printf("\nWriting Custom EDID.\n");

	// make sure to change first to the CUSTOM EDID TABLE and then upload the custom table.
	changeCustomEdid(eCustom_Uploaded_EDID);
		// note Linker will calculate the CEC physical address automatically and add to the custom EDID.
	Status = setCustomEdid(FixedEdid);
	if (Status == true) {
		printf("\nWriting EDID values...\n");
		for (int i = 0; i<256; i++) {
			if (!(i % 16))
				printf("\n");
			printf("%02X ", FixedEdid[i]);
		}
	}
	else {
		printf("\nError writing edid values.\n");
	}

	printf("\nPress ENTER key to Continue\n"); getchar();
	//*****************************************************
	printf("\n\nReading back the active EDID.\n");
	Status = getEdid(defaultEDID);
	if (Status == true) {
		for (int i = 0; i<256; i++) {
			if (!(i % 16))
				printf("\n");
			printf("%02X ", defaultEDID[i]);
		}
	}
	else {
		printf("\nError reading edid values.\n");
	}

	printf("\nPress ENTER key to Continue\n"); getchar();
	//*****************************************************

	Status = setHdrIf("87:01:17:5c:02:00:D0:84:80:3E:C2:33:C4:86:4C:1D:B8:0B:13:3D:42:40:A0:0F:05:00", 1);  // send and also check CRC
	if (Status == false)
		printf("Error Writing IF values.\n");

	setCsc(COLFMT_YUV422_BT709);  // change color space
	printf("\nColor space changed to YUV 422, Press ENTER key to Continue\n"); getchar();
	setDeepColor(DEEPC_12); // change deep color output
	printf("\nDeep color changed to 12 bits, Press ENTER key to Continue\n"); getchar();
	setPreset4k30(true);
	printf("\nPreset4k30 option enabled, Press ENTER key to Continue\n"); getchar();
	setPreset4k30Mode(PRESET_4K30_422_12);
	printf("\nChanged preset for all 4k30 input to 4k30 422 12bits, Press ENTER key to Continue\n"); getchar();
	setHdrDisable(1);
	printf("\nHDR output disabled, Press ENTER key to Continue\n"); getchar();
	setHdrDisable(0);
	printf("\nHDR output normal, Press ENTER key to Continue\n"); getchar();
	setLogoLed(false);
	printf("\nLogo LED off, Press ENTER key to Continue\n"); getchar();
	setLogoLed(true);
	printf("\nLogo LED off, Press ENTER key to Continue\n"); getchar();
	// see the rest of the commands in the linkerapi.h header


	setCsc(COLFMT_YUV444_BT709);  // change color space
	printf("\nColor space changed to YUV 444, Press ENTER key to Continue\n"); getchar();
	setCsc(COLFMT_RGB_FULL_RANGE);  // change color space
	printf("\nColor space changed to RGB FULL RANGE, Press ENTER key to Continue\n"); getchar();

	DeviceClose();  // close now USB device
	return 0;
}

bool GetLinkStatus(uint8 *RdStatus)
{
	bool St;

	St = getLinkerState(RdStatus);
	if (St == false) {
		return(St);
	}

	if (St == true) {
		switch (RdStatus[eINPUT]) {
		case INPORT_A:
			printf("\nPort A selected\n");
			break;
		case INPORT_B:
			printf("\nPort B selected\n");
			break;
		}

		switch (RdStatus[eEDID]) {
		case EDID_AUTOMIX:
			printf("EDID AUTOMIX\n");
			break;
		case EDID_FIXED:
			printf("EDID FIXED\n");
			break;
		case EDID_CUSTOM:
			printf("EDID CUSTOM\n");
			break;
		}
		switch (RdStatus[eENC]) {
		case HDCP_14:
			printf("HDCP 1.4\n");
			break;
		case HDCP_22:
			printf("HDCP 2.2\n");
			break;
		}

		switch (RdStatus[eAUTOSW]) {
		case 0x00:
			printf("Auto switching disabled\n");
			break;
		case 0x01:
			printf("Auto switching enabled\n");
			break;
		}

		if (RdStatus[eSwHdcp] == SLIDESW_MIDDLE) {
			printf("HDCP  slide switch: middle\n");
		}
		else {
			printf("HDCP  slide switch: left/right\n");
		}
		if (RdStatus[eSwEdid] == SLIDESW_MIDDLE) {
			printf("EDID  slide switch: middle\n");
		}
		else {
			printf("EDID  slide switch: left/right\n");
		}

	}
	return(St);
}
